@@ -1,5 +1,5 @@
 ; AisleRiot - eagle_wing.scm
-; Copyright (C) 1998 Rosanna Yuen <rwsy@mit.edu>
+; Copyright (C) 1998, 2003 Rosanna Yuen <rwsy@mit.edu>
 ;
 ; This game is free software; you can redistribute it and/or modify
 ; it under the terms of the GNU General Public License as published by
@@ -34,23 +34,23 @@
   (add-normal-slot '())
 
   (add-carriage-return-slot)
-  (set! VERTPOS (+ VERTPOS 100))
+  (set! VERTPOS (+ VERTPOS 0.2))
   (add-extended-slot '() down)          ;tableau (slot 6)
-  (set! VERTPOS (- VERTPOS 20))
+  (set! VERTPOS (- VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 7)
-  (set! VERTPOS (- VERTPOS 20))
+  (set! VERTPOS (- VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 8)
-  (set! VERTPOS (+ VERTPOS 20))
+  (set! VERTPOS (+ VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 9)
-  (set! VERTPOS (+ VERTPOS 50))
+  (set! VERTPOS (+ VERTPOS 0.25))
   (add-normal-slot '())                 ;reserve (slot 10)
-  (set! VERTPOS (- VERTPOS 50))
+  (set! VERTPOS (- VERTPOS 0.25))
   (add-extended-slot '() down)          ;tableau (slot 11)
-  (set! VERTPOS (- VERTPOS 20))
+  (set! VERTPOS (- VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 12)
-  (set! VERTPOS (+ VERTPOS 20))
+  (set! VERTPOS (+ VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 13)
-  (set! VERTPOS (+ VERTPOS 20))
+  (set! VERTPOS (+ VERTPOS 0.1))
   (add-extended-slot '() down)          ;tableau (slot 14)
 
   (deal-cards-face-up 0 '(10))
@@ -83,29 +83,29 @@
 					(get-redeals-string))))
 
 (define (get-stock-no-string)
-  (string-append "Stock left:  " 
+  (string-append (_"Stock left:") " "
 		 (number->string (length (get-cards 0)))))
 
 (define (get-reserve-no-string)
-  (string-append "Reserve left:  " 
+  (string-append (_"Reserve left:") " "
 		 (number->string (length (get-cards 10)))))
 
 (define (get-base-string)
   (cond ((and (> BASE-VAL 1)
 	      (< BASE-VAL 11))
-	 (string-append "Base Card:  " (number->string BASE-VAL)))
+	 (string-append (_"Base Card: ") (number->string BASE-VAL)))
 	((= BASE-VAL 1)
-	 "Base Card:  Ace")
+	 (_"Base Card: Ace"))
 	((= BASE-VAL 11)
-	 "Base Card:  Jack")
+	 (_"Base Card: Jack"))
 	((= BASE-VAL 12)
-	 "Base Card:  Queen")
+	 (_"Base Card: Queen"))
 	((= BASE-VAL 13)
-	 "Base Card:  King")
+	 (_"Base Card: King"))
 	(#t #f)))
 
 (define (get-redeals-string)
-  (string-append "Redeals left:  "
+  (string-append (_"Redeals left:") " "
 		 (number->string (- 2 FLIP-COUNTER))))
 
 (define (button-pressed slot-id card-list)
@@ -121,11 +121,7 @@
 	   (empty-slot? start-slot)
 	   (not (empty-slot? 10)))
       (deal-cards-face-up 10 (cons start-slot '())))
-  (set-statusbar-message (string-append (get-stock-no-string)
-					"   "
-					(get-reserve-no-string)
-					"   "
-					(get-base-string))))
+  (give-status-message))
 
 (define (button-released start-slot card-list end-slot)
   (and (not (= start-slot end-slot))
@@ -266,7 +262,7 @@
   (if (and (not (empty-slot? slot))
 	   (is-visible? (get-top-card slot)))
       (cond ((= (get-value (get-top-card slot)) BASE-VAL)
-	     (list 3 (get-name (get-top-card slot)) "to an empty foundation"))
+	     (list 0 (format (_"Move ~a to an empty foundation") (get-name (get-top-card slot)))))
 	    ((check-a-foundation slot 2)
 	     (list 1 
 		   (get-name (get-top-card slot)) 
@@ -299,7 +295,7 @@
 	   (not (= slot 10)))
       (if (empty-slot? 1)
 	  #f
-	  (list 2 (get-name (get-top-card 1)) "an empty slot on tableau"))
+	  (list 2 (get-name (get-top-card 1)) (_"an empty slot on tableau")))
       (if (< slot 14)
 	  (check-empty-slot (+ 1 slot))
 	  #f)))
@@ -331,10 +327,10 @@
 
 (define (dealable?)
   (if (not (empty-slot? 0))
-      (list 0 "Deal a card")
+      (list 0 (_"Deal a card"))
       (if (and (not (empty-slot? 1))
 	       (< FLIP-COUNTER 2))
-	  (list 0 "Move waste back to stock")
+	  (list 0 (_"Move waste back to stock"))
 	  #f)))
 
 (define (get-hint)
