@@ -40,6 +40,7 @@
 #include "games-frame.h"
 #include "games-files.h"
 #include "games-gridframe.h"
+#include "games-preimage.h"
 #include "glines.h"
 
 #define KEY_DIR "/apps/glines"
@@ -61,8 +62,8 @@ static GtkWidget *app, *appbar, *pref_dialog;
 GtkWidget *preview_widgets[3];
 field_props field[FIELDSIZE * FIELDSIZE];
 
-/* The unscaled pixbuf as read from file. Cached here to save file reads. */
-GdkPixbuf *raw_pixbuf = NULL;
+/* Pre-rendering image data prepared from file. */
+GamesPreimage *ball_preimage = NULL;
 /* The tile images with balls rendered on them. */
 GdkPixmap *ball_pixmap = NULL;
 /* The balls rendered to a size appropriate for the preview. */
@@ -115,10 +116,10 @@ GnomeUIInfo mainmenu[];
 
 static void 
 load_image (gchar *fname,
-	    GdkPixbuf **pixbuf)
+	    GamesPreimage **preimage)
 {
 	gchar *tmp, *fn = NULL;
-	GdkPixbuf *image;
+	GamesPreimage *tmp_preimage;
 
 	tmp = g_build_filename ("glines", fname, NULL);
 	
@@ -126,36 +127,41 @@ load_image (gchar *fname,
 	g_free (tmp);
 
 
-	if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
+	/* Just think of this as an if statement, but we want to use break. */
+	while (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
 		char * message;
 		GtkWidget * w;
-		
-		tmp = g_build_filename ("glines", "balls.svg", NULL);
-		fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
-		g_free (tmp);
-		if (!g_file_test (fn, G_FILE_TEST_EXISTS)) {
-			message = g_strdup_printf (_("Five or More could not find image file:\n%s\n\n"
-						     "Please check Five or More is installed correctly."), fn);
-			w = gtk_message_dialog_new (GTK_WINDOW (app),
-						    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-						    GTK_MESSAGE_ERROR,
-						    GTK_BUTTONS_OK,
-						    message,
-						    NULL);
-			gtk_dialog_run (GTK_DIALOG (w));	
-			g_free (message);
-			exit (1);
+
+		/* ball.png was replaced with balls.svg. */
+		if (g_utf8_collate (fname, "ball.png") == 0) {
+			tmp = g_build_filename ("glines", "balls.svg", NULL);
+			fn = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, (tmp), FALSE, NULL);
+			g_free (tmp);
+			if (g_file_test (fn, G_FILE_TEST_EXISTS))
+				break;
 		}
+
+		message = g_strdup_printf (_("Five or More could not find image file:\n%s\n\n"
+					     "Please check Five or More is installed correctly."), fn);
+		w = gtk_message_dialog_new (GTK_WINDOW (app),
+					    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					    GTK_MESSAGE_ERROR,
+					    GTK_BUTTONS_OK,
+					    message,
+					    NULL);
+		gtk_dialog_run (GTK_DIALOG (w));	
+		g_free (message);
+		exit (1);
 	}
 
-	image = gdk_pixbuf_new_from_file (fn, NULL);
+	tmp_preimage = games_preimage_new_from_uri (fn, NULL);
 	g_free( fn );
 
 
-	if (*pixbuf)
-		g_object_unref (*pixbuf);
+	if (*preimage)
+		g_object_unref (*preimage);
 
-	*pixbuf = image;
+	*preimage = tmp_preimage;
 }
 
 static void
@@ -170,9 +176,8 @@ refresh_pixmaps (void)
 	if (!ball_pixmap)
 		return;
 
-	ball_pixbuf = gdk_pixbuf_scale_simple (raw_pixbuf, 4*boxsize, 
-					       7*boxsize, 
-					       GDK_INTERP_BILINEAR);
+	ball_pixbuf = games_preimage_render (ball_preimage, 4*boxsize, 
+					     7*boxsize, NULL);
 
 	gc = gdk_gc_new (ball_pixmap);
 	gdk_gc_set_foreground (gc, &backgnd.color);
@@ -209,9 +214,8 @@ refresh_preview_pixmaps (void)
 		for (i=0; i<7; i++)
 			g_object_unref (preview_pixmaps[i]);
 
-	scaled = gdk_pixbuf_scale_simple (raw_pixbuf, 4*preview_width, 
-					  7*preview_height, 
-					  GDK_INTERP_BILINEAR);
+	scaled = games_preimage_render (ball_preimage, 4*preview_width, 
+				        7*preview_height, NULL);
 
 	for (i=0; i<7; i++) {
 		preview_pixmaps[i] = gdk_pixmap_new (widget->window,
@@ -325,7 +329,14 @@ draw_preview (void)
 }
 
 static void
-show_scores (gint pos)
+game_new_callback (GtkWidget *widget, void *data)
+{
+	reset_game ();
+ 	start_game ();
+}
+
+static void
+show_scores (gint pos, gboolean new_game)
 {
 	GtkWidget *dialog;
 
@@ -334,6 +345,10 @@ show_scores (gint pos)
 		gtk_window_set_transient_for (GTK_WINDOW (dialog),
 					      GTK_WINDOW (app));
 		gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+		if (new_game)
+			g_signal_connect (G_OBJECT (dialog), "destroy", 
+					  G_CALLBACK (game_new_callback), 
+					  NULL);
 	}
 }
 
@@ -360,7 +375,7 @@ game_over (void)
 
 	gnome_appbar_set_status (GNOME_APPBAR (appbar), _("Game Over!"));
 	pos = gnome_score_log (score, NULL, TRUE);
-	show_scores (pos);
+	show_scores (pos, TRUE);
 	update_score_state ();
 	return;
 }
@@ -919,17 +934,10 @@ animate (gpointer gp)
 	return TRUE;
 }
 
-static void
-game_new_callback (GtkWidget *widget, void *data)
-{
-	reset_game ();
- 	start_game ();
-}
-
 static void
 game_top_ten_callback(GtkWidget *widget, gpointer data)
 {
-	show_scores (0);
+	show_scores (0, FALSE);
 }
 
 static int
@@ -1046,7 +1054,7 @@ bg_color_callback (GtkWidget *widget, gpointer data)
 static void
 load_theme ()
 {
-	load_image (ball_filename, &raw_pixbuf);
+	load_image (ball_filename, &ball_preimage);
 	refresh_pixmaps ();
 	refresh_preview_pixmaps ();
 }
@@ -1211,17 +1219,13 @@ preview_configure_cb (GtkWidget * widget, GdkEventConfigure * event)
 static int
 configure_event_callback (GtkWidget *widget, GdkEventConfigure *event)
 {
-	int xboxsize, yboxsize;
-
 	if (ball_pixmap) 
 		g_object_unref (ball_pixmap);
 
 	if (blank_pixmap)
 		g_object_unref (blank_pixmap);
 
-	xboxsize = event->width/FIELDSIZE;
-	yboxsize = event->height/FIELDSIZE;
-	boxsize = MIN (xboxsize, yboxsize);
+	boxsize = (event->width - 1)/FIELDSIZE;
 
 	ball_pixmap = gdk_pixmap_new (draw_area->window, boxsize*4, boxsize*7,
 				      -1);
@@ -1571,6 +1575,7 @@ main (int argc, char *argv [])
 	g_signal_connect (G_OBJECT (draw_area), "expose_event",
 			  G_CALLBACK (field_expose_event), NULL);
 	gridframe = games_grid_frame_new (FIELDSIZE, FIELDSIZE);
+	games_grid_frame_set_padding (GAMES_GRID_FRAME(gridframe), 1, 1);
 	gtk_container_add (GTK_CONTAINER (gridframe), draw_area);
 	gtk_table_attach_defaults (GTK_TABLE (table), gridframe, 0, 1, 1, 10);
 
