@@ -27,6 +27,7 @@
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gnome.h>
+#include <games-preimage.h>
 #include "minefield.h"
 
 static struct {
@@ -110,19 +111,19 @@ static inline gint cell_idx(GtkMineField *mfield, guint x, guint y)
 
 static void _setup_sign (sign *signp, const char *file, guint minesize)
 {
-	if (signp->pixbuf == NULL) {
+	if (signp->preimage == NULL) {
 		GError *error = NULL;
-		signp->pixbuf = gdk_pixbuf_new_from_file (file, &error);
-		if (signp->pixbuf == NULL)
+		signp->preimage = games_preimage_new_from_uri (file, &error);
+		if (signp->preimage == NULL)
 			g_error (error->message);
 	} else {
 		g_object_unref (signp->scaledpixbuf);
 	}
 
-        signp->scaledpixbuf = gdk_pixbuf_scale_simple (signp->pixbuf, 
+        signp->scaledpixbuf = games_preimage_render (signp->preimage, 
 						       minesize - 2, 
 						       minesize - 2,
-						       GDK_INTERP_BILINEAR);
+						       NULL);
 	signp->width = gdk_pixbuf_get_width (signp->scaledpixbuf);
 	signp->height = gdk_pixbuf_get_height (signp->scaledpixbuf);
 }
@@ -995,9 +996,9 @@ static void gtk_minefield_init (GtkMineField *mfield)
 	mfield->started = FALSE;
 	mfield->cdown = -1;
 
-	mfield->flag.pixbuf = NULL;
-        mfield->mine.pixbuf = NULL;
-	mfield->question.pixbuf = NULL;
+	mfield->flag.preimage = NULL;
+        mfield->mine.preimage = NULL;
+	mfield->question.preimage = NULL;
 	mfield->grand = g_rand_new ();
 	mfield->thick_line = NULL;
 }
@@ -1104,7 +1105,8 @@ gint gtk_minefield_hint (GtkMineField *mfield)
 	guint *case1list, *case2list, *case3list;
 	guint *case1ptr, *case2ptr, *case3ptr;
 	gint size;
-	
+	gint retval;
+
 	g_return_val_if_fail (mfield != NULL, MINEFIELD_HINT_NO_GAME);
 	g_return_val_if_fail (GTK_IS_MINEFIELD(mfield), MINEFIELD_HINT_NO_GAME);
 	if (!mfield->in_play)
@@ -1132,14 +1134,12 @@ gint gtk_minefield_hint (GtkMineField *mfield)
 	 * Is it me ?
 	 * Or is it just pathological ?
 	 */
-	
+
 	size = mfield->xsize*mfield->ysize;
 	case1ptr = case1list = g_malloc (size*sizeof(guint));
 	case2ptr = case2list = g_malloc (size*sizeof(guint));
 	case3ptr = case3list = g_malloc (size*sizeof(guint));
 	ncase1 = ncase2 = ncase3 = 0;
-	g_return_val_if_fail (case1ptr && case2ptr && case3ptr,
-			      MINEFIELD_HINT_NO_GAME);
 	
 	for (i = 0; i<size; i++) {
 		m = mfield->mines + i;
@@ -1174,7 +1174,10 @@ gint gtk_minefield_hint (GtkMineField *mfield)
 	} else if (ncase3 > 0) {
 		a = g_rand_int_range (mfield->grand, 0, ncase3);
 		i = case3list[a];
-	} else return MINEFIELD_HINT_ALL_MINES;
+	} else {
+		retval = MINEFIELD_HINT_ALL_MINES;
+		goto cleanup;
+	}
 
 	x = i % mfield->xsize;
 	y = i / mfield->xsize;
@@ -1187,5 +1190,12 @@ gint gtk_minefield_hint (GtkMineField *mfield)
 
 	gtk_minefield_show (mfield, x, y);
 	gtk_mine_draw (mfield, x, y);
-	return MINEFIELD_HINT_ACCEPTED;
+	retval = MINEFIELD_HINT_ACCEPTED;
+
+ cleanup: 
+	g_free (case1list);
+	g_free (case2list);
+	g_free (case3list);
+
+	return retval;
 }
