@@ -19,50 +19,69 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-
 #include <config.h>
 #include <gnome.h>
+#include <gconf/gconf-client.h>
 #include <string.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+
+#include "games-preimage.h"
+#include "games-gridframe.h"
+
 #include "pieces.h"
 
 #define APPNAME "gnotski"
 #define APPNAME_LONG "GNOME Klotski"
 
-#define TILE_SIZE 34
-
 #define RELEASE 4
 #define PRESS 3
 #define MOVING 2
 #define UNUSED 1
 #define USED 0
 
+#define MINWIDTH 250
+#define MINHEIGHT 250
+
+#define THEME_TILE_CENTER 14
+#define THEME_TILE_SIZE 34
+#define THEME_OVERLAY_SIZE 8
+
+GConfClient *conf_client;
+
 GtkWidget *window;
 GtkWidget *statusbar;
 GtkWidget *space;
 GtkWidget *move_value;
+GtkWidget *outerframe;
+GtkWidget *gameframe;
 
 GdkPixmap *buffer = NULL;
-GdkPixbuf *tiles_pixmap = NULL;
+GdkPixbuf *tiles_pixbuf = NULL;
+GamesPreimage *tiles_preimage;
+
+gint space_width = 0;
+gint space_height = 0;
+
+gboolean clear_buffer = FALSE;
 
 char *map = NULL;
 char *tmpmap = NULL;
 char *move_map = NULL;
 char *orig_map = NULL;
 
+gint tile_size = 0,
+  prior_tile_size = 0;
 gint height = -1, 
-  width = -1, 
-  moves = 0;
-
-gint session_flag = 0;
+  width = -1;
+gint moves = 0;
 gint session_xpos = 0;
 gint session_ypos = 0;
-gint session_position = 0;
+
+guint draw_idle_id = 0;
 
 char current_level[16];
 
-void create_window (void);
 void create_space (void);
 void create_statusbar (void);
 
@@ -81,12 +100,13 @@ void copymap (char *, char *);
 gint mapcmp (char *, char *);
 static gint save_state (GnomeClient *, gint, GnomeRestartStyle, gint,
                         GnomeInteractStyle, gint fast, gpointer);
-void print_map (char *);
 void set_move (gint);
 void new_move (void);
 gint game_over (void);
 void game_score (void);
 
+static gboolean window_resize_cb (GtkWidget *, GdkEventConfigure *, gpointer);
+
 /* ------------------------- MENU ------------------------ */
 void new_game_cb (GtkWidget *, gpointer);
 void quit_game_cb (GtkWidget *, gpointer);
@@ -96,231 +116,194 @@ void score_cb (GtkWidget *, gpointer);
 
 GnomeUIInfo level_1_menu[] = {
   { GNOME_APP_UI_ITEM, N_("1"), NULL, level_cb,
-    "1#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdef#  " \
-    "  #ghij#  " \
-    "  #k  l#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..", 
+    "1#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdef#" \
+    "#ghij#" \
+    "#k  l#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("2"), NULL, level_cb,
-    "2#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdef#  " \
-    "  #cghi#  " \
-    "  #j  k#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..",
+    "2#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdef#" \
+    "#cghi#" \
+    "#j  k#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("3"), NULL, level_cb,
-    "3#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdde#  " \
-    "  #fghi#  " \
-    "  #j  k#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..",
+    "3#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdde#" \
+    "#fghi#" \
+    "#j  k#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("4"), NULL, level_cb,
-    "4#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdef#  " \
-    "  #cghf#  " \
-    "  #i  j#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..",
+    "4#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdef#" \
+    "#cghf#" \
+    "#i  j#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("5"), NULL, level_cb,
-    "5#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdde#  " \
-    "  #cfgh#  " \
-    "  #i  j#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..",
+    "5#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdde#" \
+    "#cfgh#" \
+    "#i  j#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("6"), NULL, level_cb,
-    "6#10#11#" \
-    "          " \
-    "          " \
-    "  ######  " \
-    "  #a**b#  " \
-    "  #a**b#  " \
-    "  #cdde#  " \
-    "  #cfge#  " \
-    "  #h  i#  " \
-    "  ##--##  " \
-    "        .." \
-    "        ..",
+    "6#6#9#" \
+    "######" \
+    "#a**b#" \
+    "#a**b#" \
+    "#cdde#" \
+    "#cfge#" \
+    "#h  i#" \
+    "##--##" \
+    "    .." \
+    "    ..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("7"), NULL, level_cb,
-    "7#10#7#" \
-    "..        " \
-    ".         " \
-    "  #####-- " \
-    "  #**aab- " \
-    "  #*ccde# " \
-    "  #fgh  # " \
-    "  ####### " ,
+    "7#7#7#" \
+    "..     " \
+    ".      " \
+    "#####--" \
+    "#**aab-" \
+    "#*ccde#" \
+    "#fgh  #" \
+    "#######" ,
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
-/* Tile Mismatch
-  { GNOME_APP_UI_ITEM, N_("8"), NULL, level_cb,
-    "8#13#11#" \
-    "  ########## " \
-    "  #a**bcccd# " \
-    "  #ee*bfghi# " \
-    "  #j****klm# " \
-    "  #opq**rst# " \
-    "  #uvw     - " \
-    "  #######--- " \
-    "         ..  " \
-    "          .  " \
-    "         ...." \
-    "           .." ,
-    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL }, */
-   GNOMEUIINFO_END
+  GNOMEUIINFO_END
 };
 
 GnomeUIInfo level_2_menu[] = {
 
   { GNOME_APP_UI_ITEM, N_("1"), NULL, level_cb,
-    "11#10#9#" \
-    "         a" \
-    " #######  " \
-    " #**bbc#  " \
-    " #defgh#  " \
-    " #ijkgh-  " \
-    " #llk  #  " \
-    " #######  " \
-    "         m" \
-    "  ..      " ,
+    "11#9#6#" \
+    "#######  " \
+    "#**bbc#  " \
+    "#defgh#  " \
+    "#ijkgh-  " \
+    "#llk  #  " \
+    "#######..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
 
   { GNOME_APP_UI_ITEM, N_("2"), NULL, level_cb,
-    "12#10#10#" \
-    "          " \
-    "  ######  " \
-    "  #abc*#  " \
-    "  # dd*#  " \
-    "  # ee*#  " \
-    "  # fgh#  " \
-    "  ##-###  " \
-    "        . " \
-    "        . " \
-    "        . ",
+    "12#6#9#" \
+    "######" \
+    "#abc*#" \
+    "# dd*#" \
+    "# ee*#" \
+    "# fgh#" \
+    "##-###" \
+    "     ." \
+    "     ." \
+    "     .",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("3"), NULL, level_cb,
-    "13#10#11#" \
-    "        .." \
-    "        . " \
-    " ####-- . " \
-    " #ab  -   " \
-    " #ccd #   " \
-    " #ccd #   " \
-    " #**ee#   " \
-    " #*fgh#   " \
-    " #*iih#   " \
-    " ######   " \
-    "          ",
+    "13#7#10#" \
+    "     .." \
+    "     . " \
+    "####-. " \
+    "#ab  - " \
+    "#ccd # " \
+    "#ccd # " \
+    "#**ee# " \
+    "#*fgh# " \
+    "#*iih# " \
+    "###### ",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("4"), NULL, level_cb,
-    "14#10#7#" \
+    "14#10#6#" \
     "  ########" \
     "  -aabc  #" \
     "  #aabdef#" \
     "  #ijggef#" \
     "  #klhh**#" \
-    "  ########" \
-    "        .." ,
+    "..########",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("5"), NULL, level_cb,
-    "15#10#9#" \
-    " .        " \
-    "..        " \
-    "  #--#### " \
-    "  #  aab# " \
-    "  # cdfb# " \
-    "  -hcefg# " \
-    "  #hijk*# " \
-    "  #hll**# " \
-    "  ####### ",
+    "15#7#9#" \
+    " .     " \
+    "..     " \
+    "#--####" \
+    "#  aab#" \
+    "# cdfb#" \
+    "-hcefg#" \
+    "#hijk*#" \
+    "#hll**#" \
+    "#######",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("6"), NULL, level_cb,
-    "16#10#8#" \
-    "  ######  " \
-    "  #abcd#  " \
-    "  #**ee#  " \
-    "  #f*g #  " \
-    "  #fh i-  " \
-    "  ####--  " \
-    "        .." \
-    "         .",
+    "16#6#8#" \
+    "######" \
+    "#abcd#" \
+    "#**ee#" \
+    "#f*g #" \
+    "#fh i-" \
+    "####--" \
+    "    .." \
+    "     .",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
-   /* Tiles mismatch
   { GNOME_APP_UI_ITEM, N_("7"), NULL, level_cb,
-    "14#13#11#" \
-    "         ... " \
-    "          .  " \
-    " ########    " \
-    " #abcdde-    " \
-    " #ffcddg-    " \
-    " #hhijkl#    " \
-    " #***jml#    " \
-    " #n*op q#    " \
-    " #nrr s #    " \
-    " ########    " \
-    "             ",
-    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL }, */
+    "17#11#8#" \
+    "########   " \
+    "#nrr s #   " \
+    "#n*op q#   " \
+    "#***jml#   " \
+    "#hhijkl#   " \
+    "#ffcddg-   " \
+    "#abcdde- . " \
+    "########...",
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
-  { GNOME_APP_UI_ITEM, N_("7"), NULL, level_cb,
-    "17#10#8#" \
-    " ######## " \
-    " #abcc**# " \
-    " #ddeef*# " \
-    " #ddghfi# " \
-    " -   jki# " \
-    " #--##### " \
-    "       .. " \
-    "        . ",
+  { GNOME_APP_UI_ITEM, N_("8"), NULL, level_cb,
+    "18#8#8#" \
+    "########" \
+    "#abcc**#" \
+    "#ddeef*#" \
+    "#ddghfi#" \
+    "-   jki#" \
+    "#--#####" \
+    "      .." \
+    "       .",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
    GNOMEUIINFO_END
@@ -352,100 +335,261 @@ GnomeUIInfo level_3_menu[] = {
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("2"), NULL, level_cb,
-    "22#11#10#" \
-    " ######### " \
-    " #abbb***# " \
-    " #abbb*c*# " \
-    " #adeefgg# " \
-    " #  eefhh# " \
-    " #    ihh# " \
-    " #    ihh# " \
-    " #---##### " \
-    "       ... " \
-    "       . . ",
+    "22#9#10#" \
+    "#########" \
+    "#abbb***#" \
+    "#abbb*c*#" \
+    "#adeefgg#" \
+    "#  eefhh#" \
+    "#    ihh#" \
+    "#    ihh#" \
+    "#---#####" \
+    "      ..." \
+    "      . .",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
-  /* Tiles mismatch
-  { GNOME_APP_UI_ITEM, N_("X"), NULL, level_cb,
-    "14#20#12#" \
-    "a     bbbbbb  cccccc" \
-    "a     bbbbbb  cccccc" \
-    "a   -##-#######     " \
-    "    -AAAAABBCC#     " \
-    "   d-   DEFGHI#     " \
-    "    #   DEFGJI#     " \
-    "  e #   KEFGLI#     " \
-    "  e #   KEFG*I#     " \
-    "    #   MM****#     " \
-    "fff ###########     " \
-    "fff  ggghhhiiiii  .j" \
-    "kkkkkggg   iiiii....",
-    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL }, 
-   */
-
   { GNOME_APP_UI_ITEM, N_("3"), NULL, level_cb,
-    "23#13#11#" \
-    "ABBBBBBBBBBBB" \
-    "AC          D" \
-    "AC######### D" \
-    " C#**abbcc# D" \
-    " C#**abbdd# D" \
-    "EC#eefgh  # D" \
-    "EC#iiijk  -FZ" \
-    "E #iiijk  -IG" \
-    "E #########IG" \
-    "EHHHH     ..G" \
-    "EHHHHJJJJJ..G",
+    "23#15#8#" \
+    "    -##-#######" \
+    "    -AAAAABBCC#" \
+    "    -   DEFGHI#" \
+    "    #   DEFGJI#" \
+    "    #   KEFGLI#" \
+    "    #   KEFG*I#" \
+    "  . #   MM****#" \
+    "....###########",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("4"), NULL, level_cb,
-    "24#11#11#" \
-    " ABCCCCCCCC" \
-    " ABCCCCCCCC" \
-    " A####### D" \
-    "EE#aab**# D" \
-    "EE#aabc*# D" \
-    "EE-defgg# D" \
-    "  #  fhh# D" \
-    "  #  ihh# D" \
-    "  #--#### D" \
-    "..FF      D" \
-    " .FFGGGGGGD",
+    "24#11#7#" \
+    "#########  " \
+    "#**abbcc#  " \
+    "#**abbdd#  " \
+    "#eefgh  #  " \
+    "#iiijk  -  " \
+    "#iiijk  -.." \
+    "#########..",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("5"), NULL, level_cb,
-    "25#8#10#" \
-    ".       " \
-    ".       " \
-    " #-#### " \
-    " # abc# " \
-    " # dec# " \
-    " #fggc# " \
-    " #fhhi# " \
-    " #fjk*# " \
-    " #flk*# " \
-    " ###### ",
+    "25#7#9#" \
+    "#######" \
+    "#aab**#" \
+    "#aabc*#" \
+    "-defgg#" \
+    "#  fhh#" \
+    "#  ihh#" \
+    "#--####" \
+    "     .." \
+    "      .",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
   { GNOME_APP_UI_ITEM, N_("6"), NULL, level_cb,
-    "26#12#12#" \
-    " ########## " \
-    " #a*bcdefg# " \
-    " #**bhhhhg# " \
-    " #*iijjkkg# " \
-    " #liimnoop# " \
-    " #qiirrr  # " \
-    " #qstuvv  # " \
-    " #qwwxvv  # " \
-    " ######--## " \
-    "           ." \
-    "          .." \
-    "          . ",
+    "26#6#10#" \
+    ".     " \
+    ".     " \
+    "#-####" \
+    "# abc#" \
+    "# dec#" \
+    "#fggc#" \
+    "#fhhi#" \
+    "#fjk*#" \
+    "#flk*#" \
+    "######",
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("7"), NULL, level_cb,
+    "27#10#12#" \
+    "##########" \
+    "#a*bcdefg#" \
+    "#**bhhhhg#" \
+    "#*iijjkkg#" \
+    "#liimnoop#" \
+    "#qiirrr  #" \
+    "#qstuvv  #" \
+    "#qwwxvv  #" \
+    "######--##" \
+    "         ." \
+    "        .." \
+    "        . ",
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("8"), NULL, level_cb,
+    "Sunshine#29#35#" \
+    "           #######           " \
+    "           # ... #           " \
+    "           #.. ..#           " \
+    "           #.   .#           " \
+    "           #.. ..#           " \
+    "           # ... #           " \
+    "############-----############" \
+    "#\x90\x90\x91\x92\x92\x80\x80\x80\x80\x80 \xb0\xb0\xb1\xb2\xb2 \x81\x81\x81\x81\x81\x98\x98\x99\x9a\x9a#" \
+    "#\x90\x90\x91\x92\x92\x80\x80\x80\x80\x80 \xb0\xb0\xb1\xb2\xb2 \x81\x81\x81\x81\x81\x98\x98\x99\x9a\x9a#" \
+    "#\x93\x93 \x94\x94\x80\x80\x80\x80\x80 \xb3\xb3 \xb4\xb4 \x81\x81\x81\x81\x81\x9b\x9b \x9c\x9c#" \
+    "#\x95\x95\x96\x97\x97\x80\x80\x80\x80\x80 \xb5\xb5\xb6\xb7\xb7 \x81\x81\x81\x81\x81\x9d\x9d\x9e\x9f\x9f#" \
+    "#\x95\x95\x96\x97\x97\x80\x80\x80\x80\x80 \xb5\xb5\xb6\xb7\xb7 \x81\x81\x81\x81\x81\x9d\x9d\x9e\x9f\x9f#" \
+    "#\x84\x84\x84\x84\x84#################\x83\x83\x83\x83\x83#" \
+    "#\x84\x84\x84\x84\x84#ddFeeA***BffOZZ#\x83\x83\x83\x83\x83#" \
+    "#\x84\x84\x84\x84\x84#ddFee** **ffOZZ#\x83\x83\x83\x83\x83#" \
+    "#\x84\x84\x84\x84\x84#MMKQQ* C *PPS\xde\xde#\x83\x83\x83\x83\x83#" \
+    "#\x84\x84\x84\x84\x84#VVLXX** **bbRcc#\x83\x83\x83\x83\x83#" \
+    "#     #VVLXXD***EbbRcc#     #" \
+    "#\xc0\xc0\xc1\xc2\xc2#\x89\x89\x89\x89\x89TTJWW\x8a\x8a\x8a\x8a\x8a#\xc8\xc8\xc9\xca\xca#" \
+    "#\xc0\xc0\xc1\xc2\xc2#\x89\x89\x89\x89\x89TTJWW\x8a\x8a\x8a\x8a\x8a#\xc8\xc8\xc9\xca\xca#" \
+    "#\xc3\xc3 \xc4\xc4#\x89\x89\x89\x89\x89GG HH\x8a\x8a\x8a\x8a\x8a#\xcb\xcb \xcc\xcc#" \
+    "#\xc5\xc5\xc6\xc7\xc7#\x89\x89\x89\x89\x89YYIgg\x8a\x8a\x8a\x8a\x8a#\xcd\xcd\xce\xcf\xcf#" \
+    "#\xc5\xc5\xc6\xc7\xc7#\x89\x89\x89\x89\x89YYIgg\x8a\x8a\x8a\x8a\x8a#\xcd\xcd\xce\xcf\xcf#" \
+    "#     #hh\xd0iilltmmpp\xd1qq#     #" \
+    "#\x85\x85\x85\x85\x85#hh\xd2iill mmpp\xddqq#\x86\x86\x86\x86\x86#" \
+    "#\x85\x85\x85\x85\x85#2y\xd3\xd4\xd5v s w\xd8\xd9x\xdaz#\x86\x86\x86\x86\x86#" \
+    "#\x85\x85\x85\x85\x85#jj\xd6kkaa nnoo\xdbrr#\x86\x86\x86\x86\x86#" \
+    "#\x85\x85\x85\x85\x85#jj\xd7kkaaunnoo\xdcrr#\x86\x86\x86\x86\x86#" \
+    "#\x85\x85\x85\x85\x85######-----######\x86\x86\x86\x86\x86#" \
+    "#\xa0\xa0\xa1\xa2\xa2\x88\x88\x88\x88\x88 \xb8\xb8\xb9\xba\xba \x87\x87\x87\x87\x87\xa8\xa8\xa9\xaa\xaa#" \
+    "#\xa0\xa0\xa1\xa2\xa2\x88\x88\x88\x88\x88 \xb8\xb8\xb9\xba\xba \x87\x87\x87\x87\x87\xa8\xa8\xa9\xaa\xaa#" \
+    "#\xa3\xa3 \xa4\xa4\x88\x88\x88\x88\x88 \xbb\xbb \xbc\xbc \x87\x87\x87\x87\x87\xab\xab \xac\xac#" \
+    "#\xa5\xa5\xa6\xa7\xa7\x88\x88\x88\x88\x88 \xbd\xbd\xbe\xbf\xbf \x87\x87\x87\x87\x87\xad\xad\xae\xaf\xaf#" \
+    "#\xa5\xa5\xa6\xa7\xa7\x88\x88\x88\x88\x88 \xbd\xbd\xbe\xbf\xbf \x87\x87\x87\x87\x87\xad\xad\xae\xaf\xaf#" \
+    "#############################",
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  GNOMEUIINFO_END
+};
+
+GnomeUIInfo level_bt_menu[] = {
+    /* 42 moves */
+  { GNOME_APP_UI_ITEM, N_("1"), NULL, level_cb,
+    "1#6#7#" \
+    "##..##" \
+    "#a..c#" \
+    "#aabc#" \
+    "#h**d#" \
+    "#g**e#" \
+    "#gfee#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("4"), NULL, level_cb,
+    "4#6#7#" \
+    "##..##" \
+    "#a..b#" \
+    "#cdde#" \
+    "#ccee#" \
+    "#f**h#" \
+    "#g**h#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("8"), NULL, level_cb,
+    "8#6#7#" \
+    "##..##" \
+    "#a..h#" \
+    "#bbgh#" \
+    "#cbff#" \
+    "#c**f#" \
+    "#d**e#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("10"), NULL, level_cb,
+    "10#6#7#" \
+    "##..##" \
+    "#a..d#" \
+    "#bcdd#" \
+    "#ccef#" \
+    "#h**f#" \
+    "#h**g#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("11"), NULL, level_cb,
+    "11#6#7#" \
+    "##..##" \
+    "#a..c#" \
+    "#bbcc#" \
+    "#ddfg#" \
+    "#d**g#" \
+    "#e**h#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+  { GNOME_APP_UI_ITEM, N_("13"), NULL, level_cb,
+    "13#6#7#" \
+    "##..##" \
+    "#a..c#" \
+    "#aabb#" \
+    "#debg#" \
+    "#d**h#" \
+    "#f**h#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+ { GNOME_APP_UI_ITEM, N_("16"), NULL, level_cb,
+    "16#6#7#" \
+    "##..##" \
+    "#a..c#" \
+    "#abcc#" \
+    "#deeh#" \
+    "#**eg#" \
+    "#**fg#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+ { GNOME_APP_UI_ITEM, N_("17"), NULL, level_cb,
+    "17#6#7#" \
+    "##..##" \
+    "#a..b#" \
+    "#ccdd#" \
+    "#ecdg#" \
+    "#e**g#" \
+    "#f**h#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+
+ { GNOME_APP_UI_ITEM, N_("19"), NULL, level_cb,
+    "19#6#7#" \
+    "##..##" \
+    "#a..d#" \
+    "#abcc#" \
+    "#**ce#" \
+    "#**gh#" \
+    "#fggh#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+
+ { GNOME_APP_UI_ITEM, N_("20"), NULL, level_cb,
+    "20#6#7#" \
+    "##..##" \
+    "#a..d#" \
+    "#abcd#" \
+    "#eegg#" \
+    "#e**g#" \
+    "#f**i#" \
+    "######" ,
+    NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
+
+                     /* Dodge i18n for the moment */
+  { GNOME_APP_UI_ITEM, "Climb Pro 24", NULL, level_cb,
+    "Climb Pro 24#9#11#" \
+    "####.####" \
+    "#aa...bb#" \
+    "#ccdddee#" \
+    "#ccfggee#" \
+    "#hhffgnn#" \
+    "#ihklmno#" \
+    "#ijklmpo#" \
+    "#jjqqqpp#" \
+    "#rrs*tuu#" \
+    "#rr***uu#" \
+    "#########",
     NULL, GNOME_APP_PIXMAP_DATA, NULL, 0, 0, NULL },
 
    GNOMEUIINFO_END
 };
 
+
 GnomeUIInfo game_menu[] = {
   { GNOME_APP_UI_SUBTREE, N_("_Novice"), NULL, level_1_menu,  NULL,NULL,
     GNOME_APP_PIXMAP_STOCK, NULL, 
@@ -454,6 +598,10 @@ GnomeUIInfo game_menu[] = {
     GNOME_APP_PIXMAP_STOCK, NULL, 
     (GdkModifierType) 0, GDK_CONTROL_MASK },
   { GNOME_APP_UI_SUBTREE, N_("_Advanced"), NULL, level_3_menu,  NULL,NULL,
+    GNOME_APP_PIXMAP_STOCK, NULL,
+    (GdkModifierType) 0, GDK_CONTROL_MASK },
+                        /* Dodge i18n for the moment. */
+  { GNOME_APP_UI_SUBTREE, "Block 10", NULL, level_bt_menu,  NULL,NULL,
     GNOME_APP_PIXMAP_STOCK, NULL,
     (GdkModifierType) 0, GDK_CONTROL_MASK },
   GNOMEUIINFO_MENU_SCORES_ITEM (score_cb, NULL),
@@ -487,6 +635,8 @@ main (int argc, char **argv)
 {
   GnomeClient *client;
 
+  int win_width, win_height;
+
   gnome_score_init (APPNAME);
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -501,13 +651,26 @@ main (int argc, char **argv)
   client = gnome_master_client ();
   g_object_ref (G_OBJECT (client));
   gtk_object_sink (GTK_OBJECT (client));
-  
   g_signal_connect (G_OBJECT (client), "save_yourself", 
                     G_CALLBACK (save_state), argv[0]);
   g_signal_connect (G_OBJECT (client), "die",
                     G_CALLBACK (quit_game_cb), argv[0]);
 
-  create_window ();
+  conf_client = gconf_client_get_default ();
+
+  window = gnome_app_new (APPNAME, N_(APPNAME_LONG));
+
+  win_width = gconf_client_get_int (conf_client, "/apps/gnotski/width", NULL);
+  win_height = gconf_client_get_int (conf_client, "/apps/gnotski/height", NULL);
+  gtk_window_set_default_size(GTK_WINDOW(window), win_width, win_height);
+  
+  gtk_widget_realize (window);
+  
+  g_signal_connect (G_OBJECT (window), "delete_event",
+                    G_CALLBACK(quit_game_cb), NULL);
+  g_signal_connect (G_OBJECT (window), "configure_event",
+                    G_CALLBACK(window_resize_cb), NULL);
+
   gnome_app_create_menus (GNOME_APP (window), main_menu);
   load_image ();
   create_space (); 
@@ -521,7 +684,6 @@ main (int argc, char **argv)
   
   gtk_main ();
   
-/*  gtk_object_unref(GTK_OBJECT(client)); */
   return 0;
 }
 
@@ -535,20 +697,10 @@ get_bg_color (void)
   return color;
 }
 
-void
-create_window (void)
-{
-  window = gnome_app_new (APPNAME, N_(APPNAME_LONG));
-  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-  gtk_widget_realize (window);
-  g_signal_connect (GTK_OBJECT (window), "delete_event", 
-                    GTK_SIGNAL_FUNC (quit_game_cb), NULL);
-}
-
-static gint
+static gboolean
 expose_space (GtkWidget *widget, GdkEventExpose *event)
 {
-  if (buffer == NULL)
+  if (buffer == NULL || clear_buffer)
     return FALSE;
 
   gdk_draw_drawable (widget->window, 
@@ -568,12 +720,12 @@ redraw_all (void)
       gui_draw_pixmap (map, x, y);
 }
 
-static gint
+static gboolean
 movable (gint id)
 {
-  if(! (id == '#' || id == '.' || id == ' ' || id == '-'))
-     return 1;
-  return 0;
+  if (id == '#' || id == '.' || id == ' ' || id == '-')
+     return FALSE;
+  return TRUE;
 }
 
 gint button_down = 0;
@@ -588,8 +740,8 @@ button_press_space (GtkWidget *widget, GdkEventButton *event)
     if (game_over ())
       return FALSE;
     button_down = 1;
-    piece_x = (gint) event->x / TILE_SIZE;
-    piece_y = (gint) event->y / TILE_SIZE;
+    piece_x = (gint) event->x / tile_size;
+    piece_y = (gint) event->y / tile_size;
     piece_id = get_piece_id (map, piece_x, piece_y); 
     copymap (move_map, map);
   }
@@ -620,8 +772,8 @@ button_motion_space (GtkWidget *widget, GdkEventButton *event)
 { 
   gint new_piece_x, new_piece_y;
   if (button_down == 1) {
-    new_piece_x = (gint) event->x / TILE_SIZE;
-    new_piece_y = (gint) event->y / TILE_SIZE;
+    new_piece_x = (gint) event->x / tile_size;
+    new_piece_y = (gint) event->y / tile_size;
     if (new_piece_x >= width || event->x < 0
         || new_piece_y >= height || event->y < 0) 
       return FALSE;
@@ -642,10 +794,12 @@ gui_draw_pixmap (char *target, gint x, gint y)
   GdkColor *bg_color;
   GtkStyle *style;
   GdkColor *fg_color;
+  
   gint value;
+  gint overlay_size;
+  gint overlay_offset;
 
   gc = space->style->black_gc;
-
   style = gtk_widget_get_style (space);
   fg_color = &style->fg[GTK_STATE_NORMAL];
 
@@ -654,31 +808,40 @@ gui_draw_pixmap (char *target, gint x, gint y)
   gdk_gc_set_foreground (gc, bg_color);
   gdk_color_free (bg_color);
   gdk_draw_rectangle (buffer, gc, TRUE,
-                      x * TILE_SIZE, y * TILE_SIZE,
-                      TILE_SIZE, TILE_SIZE);
+                      x * tile_size, y * tile_size,
+                      tile_size, tile_size);
   gdk_gc_set_foreground (gc, fg_color);
 
-  gdk_draw_pixbuf (buffer, gc, tiles_pixmap,
-                   get_piece_nr (target, x, y) * TILE_SIZE, 0, 
-                   x * TILE_SIZE, y * TILE_SIZE,
-                   TILE_SIZE, TILE_SIZE,
-                   GDK_RGB_DITHER_NORMAL, 0, 0);
-
+  if (get_piece_id (target,x,y) != ' ') {
+    gdk_draw_pixbuf (buffer, gc, tiles_pixbuf,
+                     get_piece_nr (target,x,y) * tile_size, tile_size/2, 
+                     x * tile_size, y * tile_size,
+                     tile_size, tile_size,
+                     GDK_RGB_DITHER_NORMAL, 0, 0);
+  }
+  
   if (get_piece_id (target, x, y) == '*') {
     if (get_piece_id (orig_map, x, y) == '.')
       value = 20;
     else
       value = 22;
-    gdk_draw_pixbuf (buffer, gc, tiles_pixmap,
-                     value * TILE_SIZE + 10, 10,
-                     x * TILE_SIZE + 10, y * TILE_SIZE + 10, 8, 8,
+   
+    overlay_size = THEME_OVERLAY_SIZE * tile_size / THEME_TILE_SIZE;
+    overlay_offset = THEME_TILE_CENTER * tile_size / THEME_TILE_SIZE - overlay_size / 2;
+    gdk_draw_pixbuf (buffer, gc, tiles_pixbuf,
+                     value * tile_size + overlay_offset, 
+                     overlay_offset + tile_size/2,
+                     x * tile_size + overlay_offset, 
+                     y * tile_size + overlay_offset, 
+                     overlay_size, overlay_size,
                      GDK_RGB_DITHER_NORMAL, 0, 0);
   }
 
-  gtk_widget_queue_draw_area (space, x * TILE_SIZE, y * TILE_SIZE,
-                              TILE_SIZE, TILE_SIZE);
+  gtk_widget_queue_draw_area (space, x * tile_size, y * tile_size,
+                              tile_size, tile_size);
 }
 
+
 static void
 show_score_dialog (gint pos)
 {
@@ -708,12 +871,12 @@ update_score_state (void)
   top = gnome_score_get_notable (APPNAME, current_level,
                                  &names, &scores, &scoretimes);
   if (top > 0) {
-    gtk_widget_set_sensitive (game_menu[3].widget, TRUE);
+    gtk_widget_set_sensitive (game_menu[4].widget, TRUE);
     g_strfreev (names);
     g_free (scores);
     g_free (scoretimes);
   } else {
-    gtk_widget_set_sensitive (game_menu[3].widget, FALSE);
+    gtk_widget_set_sensitive (game_menu[4].widget, FALSE);
   }
 }
 
@@ -726,40 +889,84 @@ game_score ()
   show_score_dialog (pos);
 }
 
-static gint
+static gboolean
+configure_pixmaps_idle (void)
+{
+  if (tile_size != prior_tile_size) {
+    if (tiles_pixbuf != NULL) 
+      g_object_unref (tiles_pixbuf);
+    
+    tiles_pixbuf=games_preimage_render (tiles_preimage, tile_size*27,
+                                        tile_size*2, NULL);
+    prior_tile_size = tile_size;
+  }
+
+  if (buffer != NULL)
+    g_object_unref(buffer);
+
+  buffer = gdk_pixmap_new (space->window, width * tile_size, 
+                           height * tile_size, -1);
+  clear_buffer = FALSE;
+  redraw_all();
+  draw_idle_id = 0;
+  return FALSE;
+}
+
+static void 
+configure_pixmaps (void)
+{
+  tile_size = MIN ((space_width / width), (space_height / height));
+  
+  /* Specify tile_size in multiples of 2 to handle double-height SVG theme.*/
+  if (tile_size % 2) tile_size--;
+  
+  if (clear_buffer || (tile_size != prior_tile_size)) {
+    if (!draw_idle_id)
+      draw_idle_id = g_idle_add ((GSourceFunc) configure_pixmaps_idle, NULL);
+    clear_buffer = TRUE;
+  }
+  
+  return;
+}
+
+static gboolean
 configure_space (GtkWidget *widget, GdkEventConfigure *event)
 {
-  if (width > 0) {
-    if (buffer)
-      g_object_unref (buffer);
-    buffer = gdk_pixmap_new (widget->window, widget->allocation.width, 
-                             widget->allocation.height, -1);
-    redraw_all ();
-  }
+  space_width = event -> width;
+  space_height = event -> height;
+
+  configure_pixmaps();
+ 
   return TRUE;
 }
 
 void
 create_space (void)
 {
-  gtk_widget_push_colormap (gdk_rgb_get_colormap ());
+  outerframe = gtk_aspect_frame_new (NULL,.5, .5, 1, TRUE);
+  gtk_widget_set_size_request (GTK_WIDGET(outerframe), MINWIDTH, MINHEIGHT);
+  gnome_app_set_contents (GNOME_APP(window), outerframe);
+
+  gameframe = games_grid_frame_new (9,7);
+  gtk_container_add (GTK_CONTAINER(outerframe), gameframe);
+
+  gtk_widget_push_colormap (gdk_rgb_get_colormap());
   space = gtk_drawing_area_new ();
   gtk_widget_pop_colormap ();
-  gnome_app_set_contents (GNOME_APP (window), space);
+
+  gtk_container_add (GTK_CONTAINER(gameframe),space);
   gtk_widget_set_events (space, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK
                          | GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK);
-  gtk_widget_realize (space);
-  g_signal_connect (G_OBJECT (space), "expose_event", 
+  g_signal_connect (G_OBJECT(space), "expose_event", 
                     G_CALLBACK (expose_space), NULL);
-  g_signal_connect (G_OBJECT (space), "configure_event", 
+  g_signal_connect (G_OBJECT(space), "configure_event", 
                     G_CALLBACK (configure_space), NULL);
-  g_signal_connect(G_OBJECT(space), "button_press_event", 
-                   G_CALLBACK (button_press_space), NULL);
-  g_signal_connect (G_OBJECT(space),"button_release_event",
+  g_signal_connect (G_OBJECT(space), "button_press_event", 
+                    G_CALLBACK (button_press_space), NULL);
+  g_signal_connect (G_OBJECT(space), "button_release_event",
                     G_CALLBACK (button_release_space), NULL);
   g_signal_connect (G_OBJECT(space), "motion_notify_event",
                     G_CALLBACK (button_motion_space), NULL);
-  gtk_widget_show (space);
 }
 
 void
@@ -776,8 +983,6 @@ create_statusbar (void)
   statusbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
   gtk_box_pack_end (GTK_BOX (statusbar), move_box, FALSE, FALSE, 0);
   gnome_app_set_statusbar (GNOME_APP (window), statusbar);
-
-  /*gnome_app_install_menu_hints (GNOME_APP (window), main_menu);*/
 }
 
 void
@@ -793,9 +998,9 @@ load_image (void)
   char *fname;
 
   fname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP, 
-                                     "gnotski.png", FALSE, NULL);
+                                     "gnotski.svg", FALSE, NULL);
   if (g_file_test (fname, G_FILE_TEST_EXISTS)) {
-    tiles_pixmap = gdk_pixbuf_new_from_file (fname, NULL);
+    tiles_preimage = games_preimage_new_from_uri (fname, NULL);
   } else {
     GtkWidget *dialog;
 
@@ -805,7 +1010,6 @@ load_image (void)
                                      GTK_BUTTONS_OK,
                                      _("Could not find \'%s\' pixmap file\n"),
                                      fname);
-    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     gtk_dialog_run (GTK_DIALOG (dialog));
 
     exit (1);
@@ -831,17 +1035,6 @@ new_move (void)
   g_free (str);
 }
 
-void
-print_map (char *src)
-{
-  gint x, y;
-  for (y = 0; y < height; y++) {
-    for (x = 0; x < width; x++)
-      printf ("%c", get_piece_id (src, x, y));
-    printf ("\n");
-  }
-}
-
 int
 game_over (void)
 {
@@ -977,6 +1170,7 @@ get_piece_nr (char *src, gint x, gint y)
   nr += 128 * (src[(x + 1) + (y + 1) * (width + 2)] == c);
   while (nr != image_map[i] && image_map[i] != -1) 
     i += 2;
+  if (image_map[i] == -1){printf ("nr: %i\n",nr);}
   return image_map[i + 1];
 }
 
@@ -1001,51 +1195,34 @@ static void
 prepare_map (char *level)
 {
   gint x, y, i = 0;
-  static gint first = 1;
   char tmp[32];
   char *p = level;
-
   if (p == NULL) {
-    if (first) {
-      p = \
-	".  ddjk ." \
-	"  **  hh " \
-	" **      " \
-	" *  ggff " \
-	" **   ff " \
-	"  **  ff " \
-	".  eeiff.";
-      width = 9;
-      height = 7;
-      first = 0;
-      message (_("Welcome to GNOME Klotski"));
-    } else {
-      return;
-    }
-  } else {
-    while (i < 16 && *p != '#')
-      current_level[i++] = *p++;
-    current_level[i] = '\0';
-    p++;
-    
-    i = 0; 
-    while (i < 16 && *p != '#')
-      tmp[i++] = *p++;
-    tmp[i] = '\0'; 
-    width = atoi (tmp);
-    p++;
-    
-    i = 0; 
-    while (i < 16 && *p != '#')
-      tmp[i++] = *p++;
-    tmp[i] = '\0'; 
-    height = atoi (tmp);
-    p++;
-    
-    sprintf (tmp, _("Playing level %s"), current_level);
-    
-    message (tmp);
+    p = level_1_menu[5].user_data;
   }
+    
+  while (i < 16 && *p != '#')
+    current_level[i++] = *p++;
+  current_level[i] = '\0';
+  p++;
+  
+  i = 0; 
+  while (i < 16 && *p != '#')
+    tmp[i++] = *p++;
+  tmp[i] = '\0'; 
+  width = atoi (tmp);
+  p++;
+  
+  i = 0; 
+  while (i < 16 && *p != '#')
+    tmp[i++] = *p++;
+  tmp[i] = '\0'; 
+  height = atoi (tmp);
+  p++;
+  
+  sprintf (tmp, _("Playing level %s"), current_level);
+  
+  message (tmp);
 
   if (map) {
     free (map);
@@ -1071,27 +1248,36 @@ new_game_cb (GtkWidget *widget, gpointer data)
 {
   widget = space;
 
-  prepare_map (data);
-  gtk_widget_set_size_request (GTK_WIDGET (space),
-                               width * TILE_SIZE, height * TILE_SIZE);
-  gtk_widget_realize (window);
-
+  clear_buffer = TRUE;
   set_move (0);
 
+  prepare_map (data);
+  
+  gtk_aspect_frame_set (GTK_ASPECT_FRAME(outerframe), .5, .5, (gfloat)width/(gfloat)height, FALSE);
+  games_grid_frame_set (GAMES_GRID_FRAME(gameframe), width, height);
+
+  configure_pixmaps();
+
   update_score_state ();
 }
 
 void
 quit_game_cb (GtkWidget *widget, gpointer data)
 {
-  if (buffer)
-    g_object_unref (buffer);
-  if (tiles_pixmap)
-    g_object_unref (tiles_pixmap);
-
   gtk_main_quit ();
 }
 
+static gboolean
+window_resize_cb (GtkWidget *w, GdkEventConfigure *e, gpointer data)
+{
+  gconf_client_set_int (conf_client, "/apps/gnotski/width",
+                        e->width, NULL);
+  gconf_client_set_int (conf_client, "/apps/gnotski/height",
+                        e->height, NULL);	
+
+  return FALSE;
+}
+
 static gint
 save_state (GnomeClient *client, gint phase, 
             GnomeRestartStyle save_style, gint shutdown,
@@ -1134,7 +1320,7 @@ about_cb (GtkWidget *widget, gpointer data)
   const gchar *authors[] = { "Lars Rydlinge", NULL };
   gchar *documenters[] = { "Andrew Sobala (andrew@sobala.net)", NULL };
   /* Translator credits */
-  gchar *translator_credits = _("translator_credits");
+  gchar *translator_credits = _("translator-credits");
 
   if (about != NULL) {
     gtk_window_present (GTK_WINDOW (about));
@@ -1155,11 +1341,11 @@ about_cb (GtkWidget *widget, gpointer data)
   }
   
   about = gnome_about_new (_(APPNAME_LONG), VERSION, 
-                           "Copyright \xc2\xa9 1999-2003 Lars Rydlinge",
+                           "Copyright \xc2\xa9 1999-2004 Lars Rydlinge",
                            _("A Klotski clone"),
                            (const char **)authors, 
                            (const char **)documenters,
-                           strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+                           strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
                            pixbuf);
   
   if (pixbuf != NULL)
